@@ -13,8 +13,7 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
-// LIVE.COM Streaming Media Libraries
-// Copyright (c) 1996-1999 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
 // 'Group sockets'
 // Implementation
 
@@ -197,6 +196,13 @@ Groupsock::changeDestinationParameters(struct in_addr const& newDestAddr,
 
   portNumBits destPortNum = fDests->fGroupEId.portNum();
   if (newDestPort.num() != 0) {
+    if (newDestPort.num() != destPortNum
+	&& IsMulticastAddress(destAddr.s_addr)) {
+      // Also bind to the new port number:
+      changePort(newDestPort);
+      // And rejoin the multicast group:
+      socketJoinGroup(env(), socketNum(), destAddr.s_addr);
+    }
     destPortNum = newDestPort.num();
     fDests->fPort = newDestPort;
   }
