@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2005 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
 // AMR Audio File sinks
 // Implementation
 
@@ -37,8 +37,6 @@ AMRAudioFileSink::~AMRAudioFileSink() {
 AMRAudioFileSink*
 AMRAudioFileSink::createNew(UsageEnvironment& env, char const* fileName,
 			    unsigned bufferSize, Boolean oneFilePerFrame) {
-  AMRAudioFileSink* newSink = NULL;
-
   do {
     FILE* fid;
     char const* perFrameFileNamePrefix;
@@ -53,13 +51,9 @@ AMRAudioFileSink::createNew(UsageEnvironment& env, char const* fileName,
       perFrameFileNamePrefix = NULL;
     }
 
-    newSink = new AMRAudioFileSink(env, fid, bufferSize, perFrameFileNamePrefix);
-    if (newSink == NULL) break;
-
-    return newSink;
+    return new AMRAudioFileSink(env, fid, bufferSize, perFrameFileNamePrefix);
   } while (0);
 
-  delete newSink;
   return NULL;
 }
 
