@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
 // AMR Audio RTP Sources (RFC 3267)
 // Implementation
 
@@ -93,6 +93,7 @@ private:
 private:
   // Redefined virtual functions:
   void doGetNextFrame();
+  virtual void doStopGettingFrames();
 
 private:
   FramedSource* fInputSource;
@@ -486,6 +487,10 @@ void AMRDeinterleaver::doGetNextFrame() {
   }
 }
 
+void AMRDeinterleaver::doStopGettingFrames() {
+  fInputSource->stopGettingFrames();
+}
+
 void AMRDeinterleaver
 ::afterGettingFrame(void* clientData, unsigned frameSize,
 		    unsigned /*numTruncatedBytes*/,
