@@ -14,13 +14,22 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
 // A file source that is a plain byte stream (rather than frames)
 // Implementation
 
 #if (defined(__WIN32__) || defined(_WIN32)) && !defined(_WIN32_WCE)
 #include <io.h>
 #include <fcntl.h>
+#define READ_FROM_FILES_SYNCHRONOUSLY 1
+    // Because Windows is a silly toy operating system that doesn't (reliably) treat
+    // open files as being readable sockets (which can be handled within the default
+    // "BasicTaskScheduler" event loop, using "select()"), we implement file reading
+    // in Windows using synchronous, rather than asynchronous, I/O.  This can severely
+    // limit the scalability of servers using this code that run on Windows.
+    // If this is a problem for you, then either use a better operating system,
+    // or else write your own Windows-specific event loop ("TaskScheduler" subclass)
+    // that can handle readable data in Windows open files as an event.
 #endif
 
 #include "ByteStreamFileSource.hh"
@@ -60,12 +69,12 @@ ByteStreamFileSource::createNew(UsageEnvironment& env, FILE* fid,
   return newSource;
 }
 
-void ByteStreamFileSource::seekToByteAbsolute(unsigned byteNumber) {
-  fseek(fFid, byteNumber, SEEK_SET);
+void ByteStreamFileSource::seekToByteAbsolute(u_int64_t byteNumber) {
+  SeekFile64(fFid, (int64_t)byteNumber, SEEK_SET);
 }
 
-void ByteStreamFileSource::seekToByteRelative(int offset) {
-  fseek(fFid, offset, SEEK_CUR);
+void ByteStreamFileSource::seekToByteRelative(int64_t offset) {
+  SeekFile64(fFid, offset, SEEK_CUR);
 }
 
 ByteStreamFileSource::ByteStreamFileSource(UsageEnvironment& env, FILE* fid,
@@ -74,11 +83,17 @@ ByteStreamFileSource::ByteStreamFileSource(UsageEnvironment& env, FILE* fid,
 					   unsigned playTimePerFrame)
   : FramedFileSource(env, fid), fPreferredFrameSize(preferredFrameSize),
     fPlayTimePerFrame(playTimePerFrame), fLastPlayTime(0), fFileSize(0),
-    fDeleteFidOnClose(deleteFidOnClose) {
+    fDeleteFidOnClose(deleteFidOnClose), fHaveStartedReading(False) {
 }
 
 ByteStreamFileSource::~ByteStreamFileSource() {
-  if (fDeleteFidOnClose && fFid != NULL) fclose(fFid);
+  if (fFid == NULL) return;
+
+#ifndef READ_FROM_FILES_SYNCHRONOUSLY
+  envir().taskScheduler().turnOffBackgroundReadHandling(fileno(fFid));
+#endif
+
+  if (fDeleteFidOnClose) fclose(fFid);
 }
 
 void ByteStreamFileSource::doGetNextFrame() {
@@ -87,6 +102,34 @@ void ByteStreamFileSource::doGetNextFrame() {
     return;
   }
 
+#ifdef READ_FROM_FILES_SYNCHRONOUSLY
+  doReadFromFile();
+#else
+  if (!fHaveStartedReading) {
+    // Await readable data from the file:
+    envir().taskScheduler().turnOnBackgroundReadHandling(fileno(fFid),
+	       (TaskScheduler::BackgroundHandlerProc*)&fileReadableHandler, this);
+    fHaveStartedReading = True;
+  }
+#endif
+}
+
+void ByteStreamFileSource::doStopGettingFrames() {
+#ifndef READ_FROM_FILES_SYNCHRONOUSLY
+  envir().taskScheduler().turnOffBackgroundReadHandling(fileno(fFid));
+  fHaveStartedReading = False;
+#endif
+}
+
+void ByteStreamFileSource::fileReadableHandler(ByteStreamFileSource* source, int /*mask*/) {
+  if (!source->isCurrentlyAwaitingData()) {
+    source->doStopGettingFrames(); // we're not ready for the data yet
+    return;
+  }
+  source->doReadFromFile();
+}
+
+void ByteStreamFileSource::doReadFromFile() {
   // Try to read as many bytes as will fit in the buffer provided
   // (or "fPreferredFrameSize" if less)
   if (fPreferredFrameSize > 0 && fPreferredFrameSize < fMaxSize) {
@@ -115,7 +158,14 @@ void ByteStreamFileSource::doGetNextFrame() {
     gettimeofday(&fPresentationTime, NULL);
   }
 
-  // Switch to another task, and inform the reader that he has data:
+  // Inform the reader that he has data:
+#ifdef READ_FROM_FILES_SYNCHRONOUSLY
+  // To avoid possible infinite recursion, we need to return to the event loop to do this:
   nextTask() = envir().taskScheduler().scheduleDelayedTask(0,
 				(TaskFunc*)FramedSource::afterGetting, this);
+#else
+  // Because the file read was done from the event loop, we can call the
+  // 'after getting' function directly, without risk of infinite recursion:
+  FramedSource::afterGetting(this);
+#endif
 }
