@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
 // A template for a MediaSource encapsulating an audio/video input device
 // Implementation
 
@@ -36,7 +36,6 @@ DeviceSource::~DeviceSource() {
 }
 
 void DeviceSource::doGetNextFrame() {
-
   // Arrange here for our "deliverFrame" member function to be called
   // when the next frame of data becomes available from the device.
   // This must be done in a non-blocking fashion - i.e., so that we
@@ -79,9 +78,6 @@ void DeviceSource::deliverFrame() {
 
   // Deliver the data here:
 
-  // After delivering the data, switch to another task, and inform
-  // the reader that he has data:
-  nextTask()
-    = envir().taskScheduler().scheduleDelayedTask(0, (TaskFunc*)afterGetting,
-						  this);
+  // After delivering the data, inform the reader that it is now available:
+  FramedSource::afterGetting(this);
 }
