@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2005 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
 // File sinks
 // Implementation
 
@@ -50,8 +50,6 @@ FileSink::~FileSink() {
 
 FileSink* FileSink::createNew(UsageEnvironment& env, char const* fileName,
 			      unsigned bufferSize, Boolean oneFilePerFrame) {
-  FileSink* newSink = NULL;
-
   do {
     FILE* fid;
     char const* perFrameFileNamePrefix;
@@ -66,13 +64,9 @@ FileSink* FileSink::createNew(UsageEnvironment& env, char const* fileName,
       perFrameFileNamePrefix = NULL;
     }
 
-    newSink = new FileSink(env, fid, bufferSize, perFrameFileNamePrefix);
-    if (newSink == NULL) break;
-
-    return newSink;
+    return new FileSink(env, fid, bufferSize, perFrameFileNamePrefix);
   } while (0);
 
-  delete newSink;
   return NULL;
 }
 
