@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2005 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
 // RTP sink for H.263+ video (RFC 2429)
 // Implementation
 
@@ -22,9 +22,9 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 
 H263plusVideoRTPSink
 ::H263plusVideoRTPSink(UsageEnvironment& env, Groupsock* RTPgs,
-		       unsigned char rtpPayloadFormat)
-  : VideoRTPSink(env, RTPgs, rtpPayloadFormat, 90000, "H263-1998"),
-    fAreInFragmentedFrame(False) {
+		       unsigned char rtpPayloadFormat,
+		       u_int32_t rtpTimestampFrequency)
+  : VideoRTPSink(env, RTPgs, rtpPayloadFormat, rtpTimestampFrequency, "H263-1998") {
 }
 
 H263plusVideoRTPSink::~H263plusVideoRTPSink() {
@@ -32,8 +32,9 @@ H263plusVideoRTPSink::~H263plusVideoRTPSink() {
 
 H263plusVideoRTPSink*
 H263plusVideoRTPSink::createNew(UsageEnvironment& env, Groupsock* RTPgs,
-				unsigned char rtpPayloadFormat) {
-  return new H263plusVideoRTPSink(env, RTPgs, rtpPayloadFormat);
+				unsigned char rtpPayloadFormat,
+				u_int32_t rtpTimestampFrequency) {
+  return new H263plusVideoRTPSink(env, RTPgs, rtpPayloadFormat, rtpTimestampFrequency);
 }
 
 Boolean H263plusVideoRTPSink
@@ -78,8 +79,6 @@ void H263plusVideoRTPSink
     setMarkerBit();
   }
 
-  fAreInFragmentedFrame = (numRemainingBytes > 0); // for next time
-
   // Also set the RTP timestamp:
   setTimestamp(frameTimestamp);
 }
@@ -89,5 +88,5 @@ unsigned H263plusVideoRTPSink::specialHeaderSize() const {
   // There's a 2-byte special video header.  However, if we're the first
   // (or only) fragment of a frame, then we reuse the first 2 bytes of
   // the payload instead.
-  return fAreInFragmentedFrame ? 2 : 0;
+  return (curFragmentationOffset() == 0) ? 0 : 2;
 }
