@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
 // MP3 File Sources
 // Implementation
 
@@ -49,14 +49,14 @@ MP3FileSource* MP3FileSource::createNew(UsageEnvironment& env, char const* fileN
     newSource = new MP3FileSource(env, fid);
     if (newSource == NULL) break;
 
-    unsigned fileSize = GetFileSize(fileName, fid);
+    unsigned fileSize = (unsigned)GetFileSize(fileName, fid);
     newSource->assignStream(fid, fileSize);
     if (!newSource->initializeStream()) break;
 
     return newSource;
   } while (0);
 
-  delete newSource;
+  Medium::close(newSource);
   return NULL;
 }
 
