@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
 // Demultiplexer for a MPEG 1 or 2 Program Stream
 // Implementation
 
@@ -116,7 +116,7 @@ MPEG1or2Demux::SCR::SCR()
 void MPEG1or2Demux
 ::noteElementaryStreamDeletion(MPEG1or2DemuxedElementaryStream* /*es*/) {
   if (--fNumOutstandingESs == 0 && fReclaimWhenLastESDies) {
-    delete this;
+    Medium::close(this);
   }
 }
 
@@ -206,7 +206,8 @@ Boolean MPEG1or2Demux::useSavedData(u_int8_t streamIdTag,
 
   out.isCurrentlyActive = True;
   if (afterGettingFunc != NULL) {
-    struct timeval presentationTime; // value? #####
+    struct timeval presentationTime;
+    presentationTime.tv_sec = 0; presentationTime.tv_usec = 0; // should fix #####
     (*afterGettingFunc)(afterGettingClientData, totNumBytesCopied,
 			0 /* numTruncatedBytes */, presentationTime,
 			0 /* durationInMicroseconds ?????#####*/);
