@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
 // A server demultiplexer for a MPEG 1 or 2 Program Stream
 // Implementation
 
@@ -127,7 +127,7 @@ static float MPEG1or2ProgramStreamFileDuration(UsageEnvironment& env,
     if (fileSource == NULL) break;
     dataSource = fileSource;
 
-    fileSize = fileSource->fileSize();
+    fileSize = (unsigned)(fileSource->fileSize());
     if (fileSize == 0) break;
 
     // Create a MPEG demultiplexor that reads from that source.
@@ -249,12 +249,12 @@ static void afterPlayingDummySink(DummySink* sink) {
 }
 
 static float computeSCRTimeCode(MPEG1or2Demux::SCR const& scr) {
-  float result = (float)(scr.remainingBits/90000.0 + scr.extension/300.0);
+  double result = scr.remainingBits/90000.0 + scr.extension/300.0;
   if (scr.highBit) {
     // Add (2^32)/90000 == (2^28)/5625
-    float const highBitValue = (float)((256*1024*1024)/5625.0);
+    double const highBitValue = (256*1024*1024)/5625.0;
     result += highBitValue;
   }
 
-  return result;
+  return (float)result;
 }
