@@ -1,23 +1,50 @@
+/**********
+This library is free software; you can redistribute it and/or modify it under
+the terms of the GNU Lesser General Public License as published by the
+Free Software Foundation; either version 2.1 of the License, or (at your
+option) any later version. (See <http://www.gnu.org/copyleft/lesser.html>.)
+
+This library is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
+more details.
+
+You should have received a copy of the GNU Lesser General Public License
+along with this library; if not, write to the Free Software Foundation, Inc.,
+59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+**********/
+// "liveMedia"
+// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
 // A filter that passes through (unchanged) chunks that contain an integral number
 // of MPEG-2 Transport Stream packets, but returning (in "fDurationInMicroseconds")
 // an updated estimate of the time gap between chunks.
 // Implementation
 
 #include "MPEG2TransportStreamFramer.hh"
+#include <GroupsockHelper.hh> // for "gettimeofday()"
 
 #define TRANSPORT_PACKET_SIZE 188
 #define NEW_DURATION_WEIGHT 0.5
   // How much weight to give to the latest duration measurement (must be <= 1)
+#define TIME_ADJUSTMENT_FACTOR 0.8
+  // A factor by which to adjust the duration estimate to ensure that the overall
+  // packet transmission times remains matched with the PCR times (which will be the
+  // times that we expect receivers to play the incoming packets).
+  // (must be <= 1)
+#define MAX_PLAYOUT_BUFFER_DURATION 0.1 // (seconds)
 
 ////////// PIDStatus //////////
 
 class PIDStatus {
 public:
-  PIDStatus() : lastClock(0.0), lastPacketNum(0), hasJustStarted(True) {}
+  PIDStatus(double _firstClock, double _firstRealTime)
+    : firstClock(_firstClock), lastClock(_firstClock),
+      firstRealTime(_firstRealTime), lastRealTime(_firstRealTime),
+      lastPacketNum(0) {
+  }
 
-  double lastClock;
+  double firstClock, lastClock, firstRealTime, lastRealTime;
   unsigned lastPacketNum;
-  Boolean hasJustStarted;
 };
 
 
@@ -108,8 +135,11 @@ void MPEG2TransportStreamFramer::afterGettingFrame1(unsigned frameSize,
 
   // Scan through the TS packets that we read, and update our estimate of
   // the duration of each packet:
+  struct timeval tvNow;
+  gettimeofday(&tvNow, NULL);
+  double timeNow = tvNow.tv_sec + tvNow.tv_usec/1000000.0;
   for (unsigned i = 0; i < numTSPackets; ++i) {
-    updateTSPacketDurationEstimate(&fTo[i*TRANSPORT_PACKET_SIZE]);
+    updateTSPacketDurationEstimate(&fTo[i*TRANSPORT_PACKET_SIZE], timeNow);
   }
 
   fDurationInMicroseconds
@@ -119,7 +149,8 @@ void MPEG2TransportStreamFramer::afterGettingFrame1(unsigned frameSize,
   afterGetting(this);
 }
 
-void MPEG2TransportStreamFramer::updateTSPacketDurationEstimate(unsigned char* pkt) {
+void MPEG2TransportStreamFramer
+::updateTSPacketDurationEstimate(unsigned char* pkt, double timeNow) {
   // Sanity check: Make sure we start with the sync byte:
   if (pkt[0] != TRANSPORT_SYNC_BYTE) {
     envir() << "Missing sync byte!\n";
@@ -136,6 +167,7 @@ void MPEG2TransportStreamFramer::updateTSPacketDurationEstimate(unsigned char* p
   u_int8_t const adaptation_field_length = pkt[4];
   if (adaptation_field_length == 0) return;
 
+  u_int8_t const discontinuity_indicator = pkt[5]&0x80;
   u_int8_t const pcrFlag = pkt[5]&0x10;
   if (pcrFlag == 0) return; // no PCR
 
@@ -152,28 +184,43 @@ void MPEG2TransportStreamFramer::updateTSPacketDurationEstimate(unsigned char* p
   PIDStatus* pidStatus = (PIDStatus*)(fPIDStatusTable->Lookup((char*)pid));
   if (pidStatus == NULL) {
     // We're seeing this PID's PCR for the first time:
-    pidStatus = new PIDStatus;
+    pidStatus = new PIDStatus(clock, timeNow);
     fPIDStatusTable->Add((char*)pid, pidStatus);
 #ifdef DEBUG_PCR
-    fprintf(stderr, "FIRST PCR 0x%08x+%d:%03x == %f, pkt #%lu\n", pcrBaseHigh, pkt[10]>>7, pcrExt, clock, fTSPacketCount);
+    fprintf(stderr, "PID 0x%x, FIRST PCR 0x%08x+%d:%03x == %f @ %f, pkt #%lu\n", pid, pcrBaseHigh, pkt[10]>>7, pcrExt, clock, timeNow, fTSPacketCount);
 #endif
   } else {
     // We've seen this PID's PCR before; update our per-packet duration estimate:
     double durationPerPacket
       = (clock - pidStatus->lastClock)/(fTSPacketCount - pidStatus->lastPacketNum);
-    if (pidStatus->hasJustStarted) {
+    if (fTSPacketDurationEstimate == 0.0) { // we've just started
       fTSPacketDurationEstimate = durationPerPacket;
-      pidStatus->hasJustStarted = False;
-    } else {
+    } else if (discontinuity_indicator == 0 && durationPerPacket >= 0.0) {
       fTSPacketDurationEstimate
 	= durationPerPacket*NEW_DURATION_WEIGHT
 	+ fTSPacketDurationEstimate*(1-NEW_DURATION_WEIGHT);
+
+      // Also adjust the duration estimate to try to ensure that the transmission
+      // rate matches the playout rate:
+      double transmitDuration = timeNow - pidStatus->firstRealTime;
+      double playoutDuration = clock - pidStatus->firstClock;
+      if (transmitDuration > playoutDuration) {
+	fTSPacketDurationEstimate *= TIME_ADJUSTMENT_FACTOR; // reduce estimate
+      } else if (transmitDuration + MAX_PLAYOUT_BUFFER_DURATION < playoutDuration) {
+	fTSPacketDurationEstimate /= TIME_ADJUSTMENT_FACTOR; // increase estimate
+      }
+    } else {
+      // the PCR has a discontinuity from its previous value; don't use it now,
+      // but reset our PCR and real-time values to compensate:
+      pidStatus->firstClock = clock;
+      pidStatus->firstRealTime = timeNow;
     }
 #ifdef DEBUG_PCR
-    fprintf(stderr, "PCR 0x%08x+%d:%03x == %f, pkt #%lu => this duration %f, new estimate %f\n", pcrBaseHigh, pkt[10]>>7, pcrExt, clock, fTSPacketCount, durationPerPacket, fTSPacketDurationEstimate);
+    fprintf(stderr, "PID 0x%x, PCR 0x%08x+%d:%03x == %f @ %f (diffs %f @ %f), pkt #%lu, discon %d => this duration %f, new estimate %f\n", pid, pcrBaseHigh, pkt[10]>>7, pcrExt, clock, timeNow, clock - pidStatus->firstClock, timeNow - pidStatus->firstRealTime, fTSPacketCount, discontinuity_indicator != 0, durationPerPacket, fTSPacketDurationEstimate);
 #endif
   }
 
   pidStatus->lastClock = clock;
+  pidStatus->lastRealTime = timeNow;
   pidStatus->lastPacketNum = fTSPacketCount;
 }
