@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2005 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
 // A class for generating MPEG-2 Transport Stream from one or more input
 // Elementary Stream data sources
 // Implementation
@@ -31,6 +31,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 MPEG2TransportStreamMultiplexor
 ::MPEG2TransportStreamMultiplexor(UsageEnvironment& env)
   : FramedSource(env),
+    fHaveVideoStreams(True/*by default*/),
     fOutgoingPacketCounter(0), fProgramMapVersion(0),
     fPreviousInputProgramMapVersion(0xFF), fCurrentInputProgramMapVersion(0xFF),
     fPCR_PID(0), fCurrentPID(0),
@@ -106,11 +107,14 @@ void MPEG2TransportStreamMultiplexor
     if (streamType == 0) {
       // Instead, set the stream's type to default values, based on whether
       // the stream is audio or video, and whether it's MPEG-1 or MPEG-2:
-      if ((stream_id&0xE0) == 0xC0) { // audio
-	streamType = mpegVersion == 1 ? 3 : 4;
-      } else if ((stream_id&0xF0) == 0xE0) { // video
+      if ((stream_id&0xF0) == 0xE0) { // video
 	streamType = mpegVersion == 1 ? 1 : 2;
 	if (fPCR_PID == 0) fPCR_PID = fCurrentPID; // use this stream's SCR for PCR
+      } else if ((stream_id&0xE0) == 0xC0) { // audio
+	streamType = mpegVersion == 1 ? 3 : 4;
+	if (!fHaveVideoStreams && fPCR_PID == 0) fPCR_PID = fCurrentPID; // use this stream's SCR for PCR
+      } else if (stream_id == 0xBD) { // private_stream1 (usually AC-3)
+	streamType = 0x06; // for DVB; for ATSC, use 0x81 
       } else { // something else, e.g., AC-3 uses private_stream1 (0xBD)
 	streamType = 0x81; // private 
       }
@@ -135,7 +139,8 @@ void MPEG2TransportStreamMultiplexor
     fNumTruncatedBytes = TRANSPORT_PACKET_SIZE;
   } else {
     fFrameSize = TRANSPORT_PACKET_SIZE;
-    Boolean willAddPCR = pid == fPCR_PID && startPositionInBuffer == 0;
+    Boolean willAddPCR = pid == fPCR_PID && startPositionInBuffer == 0
+      && !(fPCR.highBit == 0 && fPCR.remainingBits == 0 && fPCR.extension == 0);
     unsigned const numBytesAvailable = bufferSize - startPositionInBuffer;
     unsigned numHeaderBytes = 4; // by default
     unsigned numPCRBytes = 0; // by default
