@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
 // RTP sink for MPEG-4 Elementary Stream video (RFC 3016)
 // Implementation
 
@@ -23,8 +23,9 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 
 MPEG4ESVideoRTPSink
 ::MPEG4ESVideoRTPSink(UsageEnvironment& env, Groupsock* RTPgs,
-		      unsigned char rtpPayloadFormat)
-  : VideoRTPSink(env, RTPgs, rtpPayloadFormat, 90000, "MP4V-ES"),
+		      unsigned char rtpPayloadFormat,
+		      u_int32_t rtpTimestampFrequency)
+  : VideoRTPSink(env, RTPgs, rtpPayloadFormat, rtpTimestampFrequency, "MP4V-ES"),
     fVOPIsPresent(False), fAuxSDPLine(NULL) {
 }
 
@@ -34,8 +35,10 @@ MPEG4ESVideoRTPSink::~MPEG4ESVideoRTPSink() {
 
 MPEG4ESVideoRTPSink*
 MPEG4ESVideoRTPSink::createNew(UsageEnvironment& env, Groupsock* RTPgs,
-			       unsigned char rtpPayloadFormat) {
-  return new MPEG4ESVideoRTPSink(env, RTPgs, rtpPayloadFormat);
+			       unsigned char rtpPayloadFormat,
+			       u_int32_t rtpTimestampFrequency) {
+  return new MPEG4ESVideoRTPSink(env, RTPgs, rtpPayloadFormat,
+				 rtpTimestampFrequency);
 }
 
 Boolean MPEG4ESVideoRTPSink::sourceIsCompatibleWithUs(MediaSource& source) {
@@ -109,7 +112,7 @@ char const* MPEG4ESVideoRTPSink::auxSDPLine() {
   unsigned fmtpFmtSize = strlen(fmtpFmt)
     + 3 /* max char len */
     + 3 /* max char len */
-    + 2*configLength /* 2*, because each by prints as 2 chars */
+    + 2*configLength /* 2*, because each byte prints as 2 chars */
     + 2 /* trailing \r\n */;
   char* fmtp = new char[fmtpFmtSize];
   sprintf(fmtp, fmtpFmt, rtpPayloadType(), profile_level_id);
