@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
 // A filter that breaks up an MPEG-4 video elementary stream into
 //   frames for:
 // - Visual Object Sequence (VS) Header + Visual Object (VO) Header
@@ -80,6 +80,7 @@ private:
   unsigned fixed_vop_time_increment; // used if 'fixed_vop_rate' is set
   unsigned fSecondsSinceLastTimeCode, fTotalTicksSinceLastTimeCode, fPrevNewTotalTicks; 
   unsigned fPrevPictureCountDelta;
+  Boolean fJustSawTimeCode;
 };
 
 
@@ -134,7 +135,7 @@ void MPEG4VideoStreamFramer
 }
 
 void MPEG4VideoStreamFramer::completeNewConfig() {
-  delete fConfigBytes; fConfigBytes = fNewConfigBytes;
+  delete[] fConfigBytes; fConfigBytes = fNewConfigBytes;
   fNewConfigBytes = NULL;
   fNumConfigBytes = fNumNewConfigBytes;
   fNumNewConfigBytes = 0;
@@ -154,7 +155,7 @@ MPEG4VideoStreamParser
     vop_time_increment_resolution(0), fNumVTIRBits(0),
     fixed_vop_rate(0), fixed_vop_time_increment(0),
     fSecondsSinceLastTimeCode(0), fTotalTicksSinceLastTimeCode(0),
-    fPrevNewTotalTicks(0), fPrevPictureCountDelta(1) {
+    fPrevNewTotalTicks(0), fPrevPictureCountDelta(1), fJustSawTimeCode(False) {
 }
 
 MPEG4VideoStreamParser::~MPEG4VideoStreamParser() {
@@ -481,6 +482,7 @@ unsigned MPEG4VideoStreamParser::parseGroupOfVideoObjectPlane() {
 #if defined(DEBUG) || defined(DEBUG_TIMESTAMPS)
   fprintf(stderr, "time_code: 0x%05x, hours %d, minutes %d, marker_bit %d, seconds %d\n", time_code, time_code_hours, time_code_minutes, marker_bit, time_code_seconds);
 #endif
+  fJustSawTimeCode = True;
 
   // Now, copy all bytes that we see, up until we reach a VOP_START_CODE:
   u_int32_t next4Bytes = get4Bytes();
@@ -580,11 +582,11 @@ unsigned MPEG4VideoStreamParser::parseVideoObjectPlane() {
       fSecondsSinceLastTimeCode += modulo_time_base;
     } else {
       if (newTotalTicks < fPrevNewTotalTicks && vop_coding_type != 2/*B*/
-	  && modulo_time_base == 0 && vop_time_increment == 0) {
+	  && modulo_time_base == 0 && vop_time_increment == 0 && !fJustSawTimeCode) {
 	// This is another kind of buggy MPEG-4 video stream, in which
 	// "vop_time_increment" wraps around, but without
-	// "modulo_time_base" changing.  Overcome this by pretending that
-	// "vop_time_increment" really did wrap around:
+	// "modulo_time_base" changing (or just having had a new time code).
+	// Overcome this by pretending that "vop_time_increment" *did* wrap around:
 #ifdef DEBUG
 	fprintf(stderr, "Buggy MPEG-4 video stream: \"vop_time_increment\" wrapped around, but without \"modulo_time_base\" changing!\n");
 #endif
@@ -592,17 +594,18 @@ unsigned MPEG4VideoStreamParser::parseVideoObjectPlane() {
 	newTotalTicks += vop_time_increment_resolution;
       }
       fPrevNewTotalTicks = newTotalTicks;
-      int pictureCountDelta = newTotalTicks - fTotalTicksSinceLastTimeCode;
-      if (pictureCountDelta <= 0) pictureCountDelta = fPrevPictureCountDelta;
-          // ensures that the picture count always increases
-      usingSource()->fPictureCount += pictureCountDelta;
-      fPrevPictureCountDelta = pictureCountDelta;
-      fTotalTicksSinceLastTimeCode = newTotalTicks;
       if (vop_coding_type != 2/*B*/) {
+	int pictureCountDelta = newTotalTicks - fTotalTicksSinceLastTimeCode;
+	if (pictureCountDelta <= 0) pictureCountDelta = fPrevPictureCountDelta;
+	    // ensures that the picture count is always increasing
+	usingSource()->fPictureCount += pictureCountDelta;
+	fPrevPictureCountDelta = pictureCountDelta;
+	fTotalTicksSinceLastTimeCode = newTotalTicks;
 	fSecondsSinceLastTimeCode += modulo_time_base;
       }
     }
   }
+  fJustSawTimeCode = False; // for next time
   
   // The next thing to parse depends on the code that we just saw,
   // but we are assumed to have ended the current picture: 
@@ -616,6 +619,10 @@ unsigned MPEG4VideoStreamParser::parseVideoObjectPlane() {
     setParseState(PARSING_VISUAL_OBJECT_SEQUENCE_SEEN_CODE);
     break;
   }
+  case VISUAL_OBJECT_START_CODE: {
+    setParseState(PARSING_VISUAL_OBJECT);
+    break;
+  }
   case GROUP_VOP_START_CODE: {
     setParseState(PARSING_GROUP_OF_VIDEO_OBJECT_PLANE);
     break;
@@ -625,9 +632,20 @@ unsigned MPEG4VideoStreamParser::parseVideoObjectPlane() {
     break;
   }
   default: {
-    usingSource()->envir() << "MPEG4VideoStreamParser::parseVideoObjectPlane(): Saw unexpected code "
-			   << (void*)next4Bytes << "\n";
-    setParseState(PARSING_VIDEO_OBJECT_PLANE); // the safest way to recover...
+    if (isVideoObjectStartCode(next4Bytes)) {
+      setParseState(PARSING_VIDEO_OBJECT_LAYER);
+    } else if (isVideoObjectLayerStartCode(next4Bytes)){
+      // copy all bytes that we see, up until we reach a VOP_START_CODE:
+      u_int32_t next4Bytes = get4Bytes();
+      while (next4Bytes != VOP_START_CODE) {
+	saveToNextCode(next4Bytes);
+      }
+      setParseState(PARSING_VIDEO_OBJECT_PLANE);
+    } else {
+      usingSource()->envir() << "MPEG4VideoStreamParser::parseVideoObjectPlane(): Saw unexpected code "
+			     << (void*)next4Bytes << "\n";
+      setParseState(PARSING_VIDEO_OBJECT_PLANE); // the safest way to recover...
+    }
     break;
   }
   }
