@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2005 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
 // A data structure that represents a session that consists of
 // potentially multiple (audio and/or video) sub-sessions
 // Implementation
@@ -212,9 +212,8 @@ Boolean MediaSession::initializeWithSDP(char const* sdpDescription) {
       if (subsession->parseSDPAttribute_range(sdpLine)) continue;
       if (subsession->parseSDPAttribute_fmtp(sdpLine)) continue;
       if (subsession->parseSDPAttribute_source_filter(sdpLine)) continue;
-      if (subsession->parseSDPAttribute_x_mct_slap(sdpLine)) continue;
       if (subsession->parseSDPAttribute_x_dimensions(sdpLine)) continue;
-      if (subsession->parseSDPAttribute_x_framerate(sdpLine)) continue;
+      if (subsession->parseSDPAttribute_framerate(sdpLine)) continue;
 #ifdef SUPPORT_REAL_RTSP
       if (RealParseSDPAttributes(subsession, sdpLine)) continue;
 #endif
@@ -454,39 +453,15 @@ unsigned MediaSession::guessRTPTimestampFrequency(char const* mediumName,
   return 8000; // for "audio", and any other medium
 }
 
-static unsigned computeSeqNumStagger(unsigned staggerSeconds) {
-  // To compute the sequence number stagger, assume
-  // - one MP3 frame per packet
-  // - 1152 samples per MP3 frame (defined by MP3 standard)
-  // - a sampling frequency of 44100 Hz
-  // (Later, allow this to be parameterized)#####
-  unsigned const samplesPerFrame = 1152;
-  unsigned samplesPerSecond = 44100;
-  double secondsPerFrame = (double)samplesPerFrame/samplesPerSecond;
-  return (unsigned)(staggerSeconds/secondsPerFrame);
-}
-
 Boolean MediaSession
 ::initiateByMediaType(char const* mimeType,
 		      MediaSubsession*& resultSubsession,
-		      PrioritizedRTPStreamSelector*& resultMultiSource,
-		      int& resultMultiSourceSessionId,
 		      int useSpecialRTPoffset) {
   // Look through this session's subsessions for media that match "mimeType"
   resultSubsession = NULL;
-  resultMultiSource = NULL;
-  resultMultiSourceSessionId = 0;
-  unsigned maxStaggerSeconds = 0;
   MediaSubsessionIterator iter(*this);
   MediaSubsession* subsession;
   while ((subsession = iter.next()) != NULL) {
-    if (resultMultiSourceSessionId != 0
-	&& subsession->mctSLAPSessionId() != resultMultiSourceSessionId) {
-      // We're using a multi-source SLAP session, but this subsession
-      // isn't part of it
-      continue;
-    }
-
     Boolean wasAlreadyInitiated = subsession->readSource() != NULL;
     if (!wasAlreadyInitiated) {
       // Try to create a source for this subsession:
@@ -499,43 +474,15 @@ Boolean MediaSession
       continue;
     }
 
-    if (subsession->mctSLAPSessionId() == 0) {
-      // Normal case: a single session
-      resultSubsession = subsession;
-      break; // use this
-    } else {
-      // Special case: a multi-source SLAP session
-      resultMultiSourceSessionId = subsession->mctSLAPSessionId();
-      unsigned subsessionStaggerSeconds = subsession->mctSLAPStagger();
-      if (subsessionStaggerSeconds > maxStaggerSeconds) {
-	maxStaggerSeconds = subsessionStaggerSeconds;
-      }
-    }
+    resultSubsession = subsession;
+    break; // use this
   }
 
-  if (resultSubsession == NULL && resultMultiSourceSessionId == 0) {
+  if (resultSubsession == NULL) {
     envir().setResultMsg("Session has no usable media subsession");
     return False;
   }
 
-  if (resultMultiSourceSessionId != 0) {
-    // We have a multi-source MCT SLAP session; create a selector for it:
-    unsigned seqNumStagger = computeSeqNumStagger(maxStaggerSeconds);
-    resultMultiSource
-      = PrioritizedRTPStreamSelector::createNew(envir(), seqNumStagger);
-    if (resultMultiSource == NULL) return False;
-    // Note: each subsession has its own RTCP instance; we don't return them
-
-    // Then run through the subsessions again, adding each of the sources:
-    iter.reset();
-    while ((subsession = iter.next()) != NULL) {
-      if (subsession->mctSLAPSessionId() == resultMultiSourceSessionId) {
-	resultMultiSource->addInputRTPStream(subsession->rtpSource(),
-					     subsession->rtcpInstance());
-      }
-    }
-  }
-
   return True;
 }
 
@@ -580,7 +527,6 @@ MediaSubsession::MediaSubsession(MediaSession& parent)
     fCpresent(False), fRandomaccessindication(False),
     fConfig(NULL), fMode(NULL), fSpropParameterSets(NULL),
     fPlayEndTime(0.0),
-    fMCT_SLAP_SessionId(0), fMCT_SLAP_Stagger(0),
     fVideoWidth(0), fVideoHeight(0), fVideoFPS(0), fNumChannels(1), fScale(1.0f),
     fRTPSocket(NULL), fRTCPSocket(NULL),
     fRTPSource(NULL), fRTCPInstance(NULL), fReadSource(NULL) {
@@ -812,10 +758,6 @@ Boolean MediaSubsession::initiate(int useSpecialRTPoffset) {
 						 fRTPTimestampFrequency,
 						 mimeType);
 	delete[] mimeType;
-      } else if (strcmp(fCodecName, "X-MCT-TEXT") == 0) {
-	// A UDP-packetized text stream (*not* a RTP stream)
-	fReadSource = BasicUDPSource::createNew(env(), fRTPSocket);
-	fRTPSource = NULL; // Note!
 #ifdef SUPPORT_REAL_RTSP
       } else if (strcmp(fCodecName, "X-PN-REALAUDIO") == 0 ||
 		 strcmp(fCodecName, "X-PN-MULTIRATE-REALAUDIO-LIVE") == 0 ||
@@ -1083,7 +1025,8 @@ Boolean MediaSubsession::parseSDPAttribute_fmtp(char const* sdpLine) {
 	fObjecttype = u;
       } else if (sscanf(line, " octet-align = %u", &u) == 1) {
 	fOctetalign = u;
-      } else if (sscanf(line, " profile-level-id = %u", &u) == 1) {
+      } else if (sscanf(line, " profile-level-id = %x", &u) == 1) {
+	// Note that the "profile-level-id" parameter is assumed to be hexadecimal
 	fProfile_level_id = u;
       } else if (sscanf(line, " robust-sorting = %u", &u) == 1) {
 	fRobustsorting = u;
@@ -1145,22 +1088,6 @@ Boolean MediaSubsession
   return parseSourceFilterAttribute(sdpLine, fSourceFilterAddr);
 }
 
-Boolean MediaSubsession::parseSDPAttribute_x_mct_slap(char const* sdpLine) {
-  // Check for a "a=x-mct-slap:<slap-session-id> <stagger>" line:
-  Boolean parseSuccess = False;
-
-  int slapSessionId;
-  int slapStagger;
-  if (sscanf(sdpLine, "a=x-mct-slap: %d %d",
-	     &slapSessionId, &slapStagger) == 2) {
-    parseSuccess = True;
-    fMCT_SLAP_SessionId = slapSessionId;
-    fMCT_SLAP_Stagger = (unsigned)slapStagger;
-  }
-
-  return parseSuccess;
-}
-
 Boolean MediaSubsession::parseSDPAttribute_x_dimensions(char const* sdpLine) {
   // Check for a "a=x-dimensions:<width>,<height>" line:
   Boolean parseSuccess = False;
@@ -1175,12 +1102,16 @@ Boolean MediaSubsession::parseSDPAttribute_x_dimensions(char const* sdpLine) {
   return parseSuccess;
 }
 
-Boolean MediaSubsession::parseSDPAttribute_x_framerate(char const* sdpLine) {
-  // Check for a "a=x-framerate:<fps>" line:
+Boolean MediaSubsession::parseSDPAttribute_framerate(char const* sdpLine) {
+  // Check for a "a=framerate: <fps>" or "a=x-framerate: <fps>" line:
   Boolean parseSuccess = False;
 
+  float frate;
   int rate;
-  if (sscanf(sdpLine, "a=x-framerate:%d", &rate) == 1) {
+  if (sscanf(sdpLine, "a=framerate: %f", &frate) == 1) {
+    parseSuccess = True;
+    fVideoFPS = (unsigned)frate;
+  } else if (sscanf(sdpLine, "a=x-framerate: %d", &rate) == 1) {
     parseSuccess = True;
     fVideoFPS = (unsigned)rate;
   }
