@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
 // RTP sink for a common kind of payload format: Those which pack multiple,
 // complete codec frames (as many as possible) into each RTP packet.
 // Implementation
@@ -31,6 +31,7 @@ void MultiFramedRTPSink::setPacketSizes(unsigned preferredPacketSize,
 
   delete fOutBuf;
   fOutBuf = new OutPacketBuffer(preferredPacketSize, maxPacketSize);
+  fOurMaxPacketSize = maxPacketSize; // save value, in case subclasses need it 
 }
 
 MultiFramedRTPSink::MultiFramedRTPSink(UsageEnvironment& env,
@@ -96,8 +97,7 @@ void MultiFramedRTPSink::setMarkerBit() {
 
 void MultiFramedRTPSink::setTimestamp(struct timeval timestamp) {
   // First, convert the timestamp to a 32-bit RTP timestamp:
-  fCurrentTimestamp
-    = convertToRTPTimestamp(timestamp, isFirstPacket() && isFirstFrameInPacket());
+  fCurrentTimestamp = convertToRTPTimestamp(timestamp);
 
   // Then, insert it into the RTP packet:
   fOutBuf->insertWord(fCurrentTimestamp, fTimestampPosition);
@@ -125,6 +125,21 @@ void MultiFramedRTPSink::setFrameSpecificHeaderBytes(unsigned char const* bytes,
   fOutBuf->insert(bytes, numBytes, fCurFrameSpecificHeaderPosition + bytePosition);
 }
 
+void MultiFramedRTPSink::setFramePadding(unsigned numPaddingBytes) {
+  if (numPaddingBytes > 0) {
+    // Add the padding bytes (with the last one being the padding size):
+    unsigned char paddingBuffer[255]; //max padding
+    memset(paddingBuffer, 0, numPaddingBytes);
+    paddingBuffer[numPaddingBytes-1] = numPaddingBytes;
+    fOutBuf->enqueue(paddingBuffer, numPaddingBytes);
+
+    // Set the RTP padding bit:
+    unsigned rtpHdr = fOutBuf->extractWord(0);
+    rtpHdr |= 0x20000000;
+    fOutBuf->insertWord(rtpHdr, 0);
+  }
+}
+
 Boolean MultiFramedRTPSink::continuePlaying() {
   // Send the first packet.
   // (This will also schedule any future sends.)
@@ -278,13 +293,15 @@ void MultiFramedRTPSink
     sendPacketIfNecessary();
   } else {
     // Use this frame in our outgoing packet:
+    unsigned char * frameStart = fOutBuf->curPtr();
+    fOutBuf->increment(numFrameBytesToUse);
+        // do this now, in case "doSpecialFrameHandling()" calls "setFramePadding()" to append padding bytes
 
     // Here's where any payload format specific processing gets done:
-    doSpecialFrameHandling(curFragmentationOffset, fOutBuf->curPtr(),
+    doSpecialFrameHandling(curFragmentationOffset, frameStart,
 			   numFrameBytesToUse, presentationTime,
 			   overflowBytes);
 
-    fOutBuf->increment(numFrameBytesToUse);
     ++fNumFramesUsedSoFar;
 
     // Update the time at which the next packet should be sent, based
@@ -307,7 +324,8 @@ void MultiFramedRTPSink
         || fOutBuf->wouldOverflow(numFrameBytesToUse)
         || (fPreviousFrameEndedFragmentation && 
             !allowOtherFramesAfterLastFragment()) 
-        || !frameCanAppearAfterPacketStart(fOutBuf->curPtr(), frameSize) ) {
+        || !frameCanAppearAfterPacketStart(fOutBuf->curPtr() - frameSize,
+					   frameSize) ) {
       // The packet is ready to be sent now
       sendPacketIfNecessary();
     } else {
@@ -356,6 +374,7 @@ void MultiFramedRTPSink::sendPacketIfNecessary() {
     fOutBuf->resetPacketStart();
   }
   fOutBuf->resetOffset();
+  fNumFramesUsedSoFar = 0;
 
   if (fNoFramesLeft) {
     // We're done:
