@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2005 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
 // RTCP
 // Implementation
 
@@ -131,10 +131,16 @@ RTCPInstance::RTCPInstance(UsageEnvironment& env, Groupsock* RTCPgs,
     fHaveJustSentPacket(False), fLastPacketSentSize(0),
     fByeHandlerTask(NULL), fByeHandlerClientData(NULL),
     fSRHandlerTask(NULL), fSRHandlerClientData(NULL),
-    fRRHandlerTask(NULL), fRRHandlerClientData(NULL) {
+    fRRHandlerTask(NULL), fRRHandlerClientData(NULL),
+    fSpecificRRHandlerTable(NULL) {
 #ifdef DEBUG
   fprintf(stderr, "RTCPInstance[%p]::RTCPInstance()\n", this);
 #endif
+  if (fTotSessionBW == 0) { // not allowed!
+    env << "RTCPInstance::RTCPInstance error: totSessionBW parameter should not be zero!\n";
+    fTotSessionBW = 1;
+  }
+
   if (isSSMSource) RTCPgs->multicastSendOnly(); // don't receive multicast
     
   double timeNow = dTimeNow();
@@ -161,6 +167,11 @@ RTCPInstance::RTCPInstance(UsageEnvironment& env, Groupsock* RTCPgs,
   onExpire(this);
 }
 
+struct RRHandlerRecord {
+  TaskFunc* rrHandlerTask;
+  void* rrHandlerClientData;
+};
+
 RTCPInstance::~RTCPInstance() {
 #ifdef DEBUG
   fprintf(stderr, "RTCPInstance[%p]::~RTCPInstance()\n", this);
@@ -173,6 +184,15 @@ RTCPInstance::~RTCPInstance() {
   fTypeOfEvent = EVENT_BYE; // not used, but...
   sendBYE();
 
+  if (fSpecificRRHandlerTable != NULL) {
+    AddressPortLookupTable::Iterator iter(*fSpecificRRHandlerTable);
+    RRHandlerRecord* rrHandler;
+    while ((rrHandler = (RRHandlerRecord*)iter.next()) != NULL) {
+      delete rrHandler;
+    }
+    delete fSpecificRRHandlerTable;
+  }
+
   delete fKnownMembers;
   delete fOutBuf;
   delete[] fInBuf;
@@ -231,6 +251,35 @@ void RTCPInstance::setRRHandler(TaskFunc* handlerTask, void* clientData) {
   fRRHandlerClientData = clientData;
 }
 
+void RTCPInstance
+::setSpecificRRHandler(netAddressBits fromAddress, Port fromPort,
+		       TaskFunc* handlerTask, void* clientData) {
+  if (handlerTask == NULL && clientData == NULL) {
+    unsetSpecificRRHandler(fromAddress, fromPort);
+    return;
+  }
+
+  RRHandlerRecord* rrHandler = new RRHandlerRecord;
+  rrHandler->rrHandlerTask = handlerTask;
+  rrHandler->rrHandlerClientData = clientData;
+  if (fSpecificRRHandlerTable == NULL) {
+    fSpecificRRHandlerTable = new AddressPortLookupTable;
+  }
+  fSpecificRRHandlerTable->Add(fromAddress, (~0), fromPort, rrHandler);
+}
+
+void RTCPInstance
+::unsetSpecificRRHandler(netAddressBits fromAddress, Port fromPort) {
+  if (fSpecificRRHandlerTable == NULL) return;
+
+  RRHandlerRecord* rrHandler
+    = (RRHandlerRecord*)(fSpecificRRHandlerTable->Lookup(fromAddress, (~0), fromPort));
+  if (rrHandler != NULL) {
+    fSpecificRRHandlerTable->Remove(fromAddress, (~0), fromPort);
+    delete rrHandler;
+  }
+}
+
 void RTCPInstance::setStreamSocket(int sockNum,
 				   unsigned char streamChannelId) {
   // Turn off background read handling:
@@ -273,6 +322,8 @@ void RTCPInstance::incomingReportHandler1() {
   int typeOfPacket = PACKET_UNKNOWN_TYPE;
 
   do {
+    int tcpReadStreamSocketNum = fRTCPInterface.nextTCPReadStreamSocketNum();
+    unsigned char tcpReadStreamChannelId = fRTCPInterface.nextTCPReadStreamChannelId();
     if (!fRTCPInterface.handleRead(pkt, maxPacketSize,
 				   packetSize, fromAddress)) {
       break;
@@ -402,6 +453,32 @@ void RTCPInstance::incomingReportHandler1() {
 
 	  if (pt == RTCP_PT_RR) { // i.e., we didn't fall through from 'SR'
 	    // If a 'RR handler' was set, call it now:
+
+	    // Specific RR handler:
+	    if (fSpecificRRHandlerTable != NULL) {
+	      netAddressBits fromAddr;
+	      portNumBits fromPortNum;
+	      if (tcpReadStreamSocketNum < 0) {
+		// Normal case: We read the RTCP packet over UDP
+		fromAddr = fromAddress.sin_addr.s_addr;
+		fromPortNum = ntohs(fromAddress.sin_port);
+	      } else {
+		// Special case: We read the RTCP packet over TCP (interleaved)
+		// Hack: Use the TCP socket and channel id to look up the handler
+		fromAddr = tcpReadStreamSocketNum;
+		fromPortNum = tcpReadStreamChannelId;
+	      }
+	      Port fromPort(fromPortNum); 
+	      RRHandlerRecord* rrHandler
+		= (RRHandlerRecord*)(fSpecificRRHandlerTable->Lookup(fromAddr, (~0), fromPort));
+	      if (rrHandler != NULL) {
+		if (rrHandler->rrHandlerTask != NULL) {
+		  (*(rrHandler->rrHandlerTask))(rrHandler->rrHandlerClientData);
+		}
+	      }
+	    }
+
+	    // General RR handler:
 	    if (fRRHandlerTask != NULL) (*fRRHandlerTask)(fRRHandlerClientData);
 	  }
 
@@ -506,6 +583,11 @@ void RTCPInstance::onReceive(int typeOfPacket, int totPacketSize,
 }
 
 void RTCPInstance::sendReport() {
+  // Hack: Don't send a SR during those (brief) times when the timestamp of the
+  // next outgoing RTP packet has been preset, to ensure that that timestamp gets
+  // used for that outgoing packet. (David Bertrand, 2006.07.18)
+  if (fSink != NULL && fSink->nextTimestampHasBeenPreset()) return;
+
 #ifdef DEBUG
   fprintf(stderr, "sending REPORT\n");
 #endif
