@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2005 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
 // A generic RTSP client
 // Implementation
 
@@ -143,8 +143,9 @@ void RTSPClient::setUserAgentString(char const* userAgentStr) {
 }
 
 RTSPClient::~RTSPClient() {
+  envir().taskScheduler().turnOffBackgroundReadHandling(fInputSocketNum); // must be called before:
   reset();
-  envir().taskScheduler().turnOffBackgroundReadHandling(fInputSocketNum);
+
   delete[] fResponseBuffer;
   delete[] fUserAgentHeaderStr;
 }
@@ -200,7 +201,7 @@ char* RTSPClient::describeURL(char const* url, Authenticator* authenticator,
     char* username; char* password;
     if (authenticator == NULL
 	&& parseRTSPURLUsernamePassword(url, username, password)) {
-      char* result = describeWithPassword(url, username, password);
+      char* result = describeWithPassword(url, username, password, allowKasennaProtocol);
       delete[] username; delete[] password; // they were dynamically allocated
       return result;
     }
@@ -268,11 +269,13 @@ char* RTSPClient::describeURL(char const* url, Authenticator* authenticator,
       break;
     }
 
-    // Skip every subsequent header line, until we see a blank line
+    // Skip over subsequent header lines, until we see a blank line.
     // The remaining data is assumed to be the SDP descriptor that we want.
-    // We should really do some checking on the headers here - e.g., to
-    // check for "Content-type: application/sdp", "Content-base",
-    // "Content-location", "CSeq", etc. #####
+    // While skipping over the header lines, we also check for certain headers
+    // that we recognize.
+    // (We should also check for "Content-type: application/sdp",
+    // "Content-location", "CSeq", etc.) #####
+    char* contentBase = new char[fResponseBufferSize]; // ensures enough space
     char* serverType = new char[fResponseBufferSize]; // ensures enough space
     int contentLength = -1;
     char* lineStart;
@@ -290,6 +293,10 @@ char* RTSPClient::describeURL(char const* url, Authenticator* authenticator,
 			       lineStart, "\"");
 	  break;
 	}
+      } else if (sscanf(lineStart, "Content-Base: %s", contentBase) == 1) {
+	if (contentBase[0] != '\0'/*sanity check*/) {
+	  delete[] fBaseURL; fBaseURL = strDup(contentBase);
+	}
       } else if (sscanf(lineStart, "Server: %s", serverType) == 1) {
 	if (strncmp(serverType, "Kasenna", 7) == 0) fServerIsKasenna = True;
 	if (strncmp(serverType, "WMServer", 8) == 0) fServerIsMicrosoft = True;
@@ -304,7 +311,7 @@ char* RTSPClient::describeURL(char const* url, Authenticator* authenticator,
 		    << redirectionURL << "\"\n";
 	  }
 	  reset();
-	  char* result = describeURL(redirectionURL);
+	  char* result = describeURL(redirectionURL, authenticator, allowKasennaProtocol);
 	  delete[] redirectionURL;
 	  delete[] serverType;
 	  return result;
@@ -312,6 +319,7 @@ char* RTSPClient::describeURL(char const* url, Authenticator* authenticator,
       }
     } 
     delete[] serverType;
+    delete[] contentBase;
 
     // We're now at the end of the response header lines
     if (wantRedirection) {
@@ -490,10 +498,11 @@ char* RTSPClient::describeURL(char const* url, Authenticator* authenticator,
 
 char* RTSPClient
 ::describeWithPassword(char const* url,
-		       char const* username, char const* password) {
+		       char const* username, char const* password,
+		       Boolean allowKasennaProtocol) {
   Authenticator authenticator;
   authenticator.setUsernameAndPassword(username, password);
-  char* describeResult = describeURL(url, &authenticator);
+  char* describeResult = describeURL(url, &authenticator, allowKasennaProtocol);
   if (describeResult != NULL) {
     // We are already authorized
     return describeResult;
@@ -506,7 +515,7 @@ char* RTSPClient
   }
 
   // Try again:
-  describeResult = describeURL(url, &authenticator);
+  describeResult = describeURL(url, &authenticator, allowKasennaProtocol);
   if (describeResult != NULL) {
     // The authenticator worked, so use it in future requests:
     fCurrentAuthenticator = authenticator;
@@ -1946,12 +1955,12 @@ RTSPClient::createAuthenticatorString(Authenticator const* authenticator,
       authenticator->reclaimDigestResponse(response);
     } else { // Basic authentication
       char* const authFmt = "Authorization: Basic %s\r\n";
-      char* usernamePassword
-	= new char[strlen(authenticator->username())
-		  + strlen(authenticator->password()) + 2];
-      sprintf(usernamePassword, "%s:%s",
-	      authenticator->username(), authenticator->password());
-      char* response = base64Encode(usernamePassword);
+
+      unsigned usernamePasswordLength = strlen(authenticator->username()) + 1 + strlen(authenticator->password());
+      char* usernamePassword = new char[usernamePasswordLength+1];
+      sprintf(usernamePassword, "%s:%s", authenticator->username(), authenticator->password());
+
+      char* response = base64Encode(usernamePassword, usernamePasswordLength);
       unsigned authBufSize = strlen(authFmt) + strlen(response);
       authenticatorStr = new char[authBufSize];
       sprintf(authenticatorStr, authFmt, response);
@@ -2006,7 +2015,7 @@ Boolean RTSPClient::sendRequest(char const* requestString, char const* tag,
 
   char* newRequestString = NULL;
   if (fTunnelOverHTTPPortNum != 0 && base64EncodeIfOverHTTP) {
-    requestString = newRequestString = base64Encode(requestString);
+    requestString = newRequestString = base64Encode(requestString, strlen(requestString));
     if (fVerbosityLevel >= 1) {
       envir() << "\tThe request was base-64 encoded to: " << requestString << "\n\n";
     }
@@ -2439,6 +2448,7 @@ void RTSPClient::incomingRequestHandler1() {
   bytesRead = getResponse1(readBuf, fResponseBufferSize);
   if (bytesRead == 0) {
     envir().setResultErrMsg("Failed to read response: ");
+    envir().taskScheduler().turnOffBackgroundReadHandling(fInputSocketNum); // because the connection died
     return;
   }
   // Parse the request string into command name and 'CSeq',
