@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2005 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
 // A RTSP server
 // Implementation
 
@@ -99,31 +99,34 @@ void RTSPServer::removeServerMediaSession(char const* streamName) {
   removeServerMediaSession(lookupServerMediaSession(streamName));
 }
 
-char* RTSPServer
-::rtspURL(ServerMediaSession const* serverMediaSession) const {
+char* RTSPServer::rtspURLPrefix() const {
   struct in_addr ourAddress;
   ourAddress.s_addr = ReceivingInterfaceAddr != 0
     ? ReceivingInterfaceAddr
     : ourSourceAddressForMulticast(envir()); // hack
 
-  char const* sessionName = serverMediaSession->streamName();
-  unsigned sessionNameLength = strlen(sessionName);
-
-  char* urlBuffer = new char[100 + sessionNameLength];
-  char* resultURL;
+  char urlBuffer[100]; // more than big enough for "rtsp://<ip-address>:<port>/"
 
   portNumBits portNumHostOrder = ntohs(fServerPort.num());
   if (portNumHostOrder == 554 /* the default port number */) {
-    sprintf(urlBuffer, "rtsp://%s/%s", our_inet_ntoa(ourAddress),
-	    sessionName);
+    sprintf(urlBuffer, "rtsp://%s/", our_inet_ntoa(ourAddress));
   } else {
-    sprintf(urlBuffer, "rtsp://%s:%hu/%s",
-	    our_inet_ntoa(ourAddress), portNumHostOrder,
-	    sessionName);
+    sprintf(urlBuffer, "rtsp://%s:%hu/",
+	    our_inet_ntoa(ourAddress), portNumHostOrder);
   }
 
-  resultURL = strDup(urlBuffer);
-  delete[] urlBuffer;
+  return strDup(urlBuffer);
+}
+
+char* RTSPServer
+::rtspURL(ServerMediaSession const* serverMediaSession) const {
+  char* urlPrefix = rtspURLPrefix();
+  char const* sessionName = serverMediaSession->streamName();
+
+  char* resultURL = new char[strlen(urlPrefix) + strlen(sessionName) + 1];
+  sprintf(resultURL, "%s%s", urlPrefix, sessionName);
+
+  delete[] urlPrefix;
   return resultURL;
 }
 
@@ -217,6 +220,7 @@ void RTSPServer::incomingConnectionHandler1() {
     }
     return;
   }
+  makeSocketNonBlocking(clientSocket);
 #if defined(DEBUG) || defined(DEBUG_CONNECTIONS)
   fprintf(stderr, "accept()ed connection from %s\n", our_inet_ntoa(clientAddr.sin_addr));
 #endif
@@ -235,9 +239,11 @@ RTSPServer::RTSPClientSession
   : fOurServer(ourServer), fOurSessionId(sessionId),
     fOurServerMediaSession(NULL),
     fClientSocket(clientSocket), fClientAddr(clientAddr),
-    fLivenessCheckTask(NULL), fSessionIsActive(True), fStreamAfterSETUP(False),
+    fLivenessCheckTask(NULL),
+    fIsMulticast(False), fSessionIsActive(True), fStreamAfterSETUP(False),
     fTCPStreamIdCount(0), fNumStreamStates(0), fStreamStates(NULL) {
   // Arrange to handle incoming requests:
+  resetRequestBuffer();
   envir().taskScheduler().turnOnBackgroundReadHandling(fClientSocket,
      (TaskScheduler::BackgroundHandlerProc*)&incomingRequestHandler, this);
   noteLiveness();
@@ -274,6 +280,12 @@ void RTSPServer::RTSPClientSession::reclaimStreamStates() {
   fNumStreamStates = 0;
 }
 
+void RTSPServer::RTSPClientSession::resetRequestBuffer() {
+  fRequestBytesAlreadySeen = 0;
+  fRequestBufferBytesLeft = sizeof fRequestBuffer;
+  fLastCRLF = &fRequestBuffer[-3]; // hack
+}
+
 void RTSPServer::RTSPClientSession
 ::incomingRequestHandler(void* instance, int /*mask*/) {
   RTSPClientSession* session = (RTSPClientSession*)instance;
@@ -284,58 +296,53 @@ void RTSPServer::RTSPClientSession::incomingRequestHandler1() {
   noteLiveness();
 
   struct sockaddr_in dummy; // 'from' address, meaningless in this case
-  int bytesLeft = sizeof fBuffer;
-  int totalBytes = 0;
   Boolean endOfMsg = False;
-  unsigned char* ptr = fBuffer;
-  unsigned char* lastCRLF = ptr-3;
+  unsigned char* ptr = &fRequestBuffer[fRequestBytesAlreadySeen];
   
-  while (!endOfMsg) {
-    if (bytesLeft <= 0) {
-      // command too big
-      delete this;
-      return;
-    }
-    
-    int bytesRead = readSocket(envir(), fClientSocket,
-			       ptr, bytesLeft, dummy);
-    if (bytesRead <= 0) {
-      // The client socket has apparently died - kill it:
-      delete this;
-      return;
-    }
+  int bytesRead = readSocket(envir(), fClientSocket,
+			     ptr, fRequestBufferBytesLeft, dummy);
+  if (bytesRead <= 0 || (unsigned)bytesRead >= fRequestBufferBytesLeft) {
+    // Either the client socket has died, or the request was too big for us.
+    // Terminate this connection:
 #ifdef DEBUG
-    ptr[bytesRead] = '\0';
-    fprintf(stderr, "RTSPClientSession[%p]::incomingRequestHandler1() read %d bytes:%s\n", this, bytesRead, ptr);
+    fprintf(stderr, "RTSPClientSession[%p]::incomingRequestHandler1() read %d bytes (of %d); terminating connection!\n", this, bytesRead, fRequestBufferBytesLeft);
+#endif
+    delete this;
+    return;
+  }
+#ifdef DEBUG
+  ptr[bytesRead] = '\0';
+  fprintf(stderr, "RTSPClientSession[%p]::incomingRequestHandler1() read %d bytes:%s\n", this, bytesRead, ptr);
 #endif
 
-    // Look for the end of the message: <CR><LF><CR><LF>
-    unsigned char *tmpPtr = ptr;
-    if (totalBytes > 0) --tmpPtr; // In case the last read ended with a <CR>
-    while (tmpPtr < &ptr[bytesRead-1]) {
-      if (*tmpPtr == '\r' && *(tmpPtr+1) == '\n') {
-	if (tmpPtr - lastCRLF == 2) { // This is it:
-	  endOfMsg = 1;
-	  break;
-	}
-	lastCRLF = tmpPtr;
+  // Look for the end of the message: <CR><LF><CR><LF>
+  unsigned char *tmpPtr = ptr;
+  if (fRequestBytesAlreadySeen > 0) --tmpPtr;
+      // in case the last read ended with a <CR>
+  while (tmpPtr < &ptr[bytesRead-1]) {
+    if (*tmpPtr == '\r' && *(tmpPtr+1) == '\n') {
+      if (tmpPtr - fLastCRLF == 2) { // This is it:
+	endOfMsg = 1;
+	break;
       }
-      ++tmpPtr;
+      fLastCRLF = tmpPtr;
     }
-  
-    bytesLeft -= bytesRead;
-    totalBytes += bytesRead;
-    ptr += bytesRead;
+    ++tmpPtr;
   }
-  fBuffer[totalBytes] = '\0';
+  
+  fRequestBufferBytesLeft -= bytesRead;
+  fRequestBytesAlreadySeen += bytesRead;
+
+  if (!endOfMsg) return; // subsequent reads will be needed to complete the request
 
   // Parse the request string into command name and 'CSeq',
   // then handle the command:
+  fRequestBuffer[fRequestBytesAlreadySeen] = '\0';
   char cmdName[RTSP_PARAM_STRING_MAX];
   char urlPreSuffix[RTSP_PARAM_STRING_MAX];
   char urlSuffix[RTSP_PARAM_STRING_MAX];
   char cseq[RTSP_PARAM_STRING_MAX];
-  if (!parseRTSPRequestString((char*)fBuffer, totalBytes,
+  if (!parseRTSPRequestString((char*)fRequestBuffer, fRequestBytesAlreadySeen,
 			      cmdName, sizeof cmdName,
 			      urlPreSuffix, sizeof urlPreSuffix,
 			      urlSuffix, sizeof urlSuffix,
@@ -351,15 +358,15 @@ void RTSPServer::RTSPClientSession::incomingRequestHandler1() {
     if (strcmp(cmdName, "OPTIONS") == 0) {
       handleCmd_OPTIONS(cseq);
     } else if (strcmp(cmdName, "DESCRIBE") == 0) {
-      handleCmd_DESCRIBE(cseq, urlSuffix, (char const*)fBuffer);
+      handleCmd_DESCRIBE(cseq, urlSuffix, (char const*)fRequestBuffer);
     } else if (strcmp(cmdName, "SETUP") == 0) {
-      handleCmd_SETUP(cseq, urlPreSuffix, urlSuffix, (char const*)fBuffer);
+      handleCmd_SETUP(cseq, urlPreSuffix, urlSuffix, (char const*)fRequestBuffer);
     } else if (strcmp(cmdName, "TEARDOWN") == 0
 	       || strcmp(cmdName, "PLAY") == 0
 	       || strcmp(cmdName, "PAUSE") == 0
 	       || strcmp(cmdName, "GET_PARAMETER") == 0) {
       handleCmd_withinSession(cmdName, urlPreSuffix, urlSuffix, cseq,
-			      (char const*)fBuffer);
+			      (char const*)fRequestBuffer);
     } else {
       handleCmd_notSupported(cseq);
     }
@@ -374,8 +381,10 @@ void RTSPServer::RTSPClientSession::incomingRequestHandler1() {
     // The client has asked for streaming to commence now, rather than after a
     // subsequent "PLAY" command.  So, simulate the effect of a "PLAY" command:
     handleCmd_withinSession("PLAY", urlPreSuffix, urlSuffix, cseq,
-			    (char const*)fBuffer);
+			    (char const*)fRequestBuffer);
   }
+
+  resetRequestBuffer(); // to prepare for any subsequent request
   if (!fSessionIsActive) delete this;
 }
 
@@ -599,6 +608,17 @@ static Boolean parseRangeHeader(char const* buf, float& rangeStart, float& range
   return True;
 }
 
+static Boolean parsePlayNowHeader(char const* buf) {
+  // Find "x-playNow:" header, if present
+  while (1) {
+    if (*buf == '\0') return False; // not found
+    if (_strncasecmp(buf, "x-playNow:", 10) == 0) break;
+    ++buf;
+  }
+
+  return True;
+}
+
 void RTSPServer::RTSPClientSession
 ::handleCmd_SETUP(char const* cseq,
 		  char const* urlPreSuffix, char const* urlSuffix,
@@ -698,7 +718,8 @@ void RTSPServer::RTSPClientSession
   // Next, check whether a "Range:" header is present in the request.
   // This isn't legal, but some clients do this to combine "SETUP" and "PLAY":
   float rangeStart, rangeEnd;
-  fStreamAfterSETUP = parseRangeHeader(fullRequestStr, rangeStart, rangeEnd);
+  fStreamAfterSETUP = parseRangeHeader(fullRequestStr, rangeStart, rangeEnd) ||
+                      parsePlayNowHeader(fullRequestStr);
 
   // Then, get server parameters from the 'subsession':
   int tcpSocketNum = streamingMode == RTP_TCP ? fClientSocket : -1;
@@ -716,32 +737,49 @@ void RTSPServer::RTSPClientSession
   destinationTTL = clientsDestinationTTL;
 #endif
   delete[] clientsDestinationAddressStr;
-  Boolean isMulticast;
   Port serverRTPPort(0);
   Port serverRTCPPort(0);
   subsession->getStreamParameters(fOurSessionId, fClientAddr.sin_addr.s_addr,
 				  clientRTPPort, clientRTCPPort,
 				  tcpSocketNum, rtpChannelId, rtcpChannelId,
-				  destinationAddress, destinationTTL, isMulticast,
+				  destinationAddress, destinationTTL, fIsMulticast,
 				  serverRTPPort, serverRTCPPort,
 				  fStreamStates[streamNum].streamToken);
   struct in_addr destinationAddr; destinationAddr.s_addr = destinationAddress;
-  if (isMulticast) {
-    if (streamingMode == RTP_TCP) {
-      // multicast streams can't be sent via TCP
-      handleCmd_unsupportedTransport(cseq);
-      return;
+  if (fIsMulticast) {
+    switch (streamingMode) {
+    case RTP_UDP:
+        snprintf((char*)fResponseBuffer, sizeof fResponseBuffer,
+                 "RTSP/1.0 200 OK\r\n"
+                 "CSeq: %s\r\n"
+                 "%s"
+                 "Transport: RTP/AVP;multicast;destination=%s;port=%d-%d;ttl=%d\r\n"
+                 "Session: %d\r\n\r\n",
+                 cseq,
+                 dateHeader(),
+                 our_inet_ntoa(destinationAddr), ntohs(serverRTPPort.num()), ntohs(serverRTCPPort.num()), destinationTTL,
+                 fOurSessionId);
+        break;
+    case RTP_TCP:
+        // multicast streams can't be sent via TCP
+        handleCmd_unsupportedTransport(cseq);
+        break;
+    case RAW_UDP:
+        snprintf((char*)fResponseBuffer, sizeof fResponseBuffer,
+                 "RTSP/1.0 200 OK\r\n"
+                 "CSeq: %s\r\n"
+                 "%s"
+                 "Transport: %s;multicast;destination=%s;client_port=%d;server_port=%d\r\n"
+                 "Session: %d\r\n\r\n",
+                 cseq,
+                 dateHeader(),
+                 streamingModeString,
+                 our_inet_ntoa(destinationAddr),
+                 ntohs(clientRTPPort.num()), ntohs(serverRTPPort.num()),
+                 fOurSessionId);
+        delete[] streamingModeString;
+        break;
     }
-    snprintf((char*)fResponseBuffer, sizeof fResponseBuffer,
-	     "RTSP/1.0 200 OK\r\n"
-	     "CSeq: %s\r\n"
-	     "%s"
-	     "Transport: RTP/AVP;multicast;destination=%s;port=%d-%d;ttl=%d\r\n"
-	     "Session: %d\r\n\r\n",
-	     cseq,
-	     dateHeader(),
-	     our_inet_ntoa(destinationAddr), ntohs(serverRTPPort.num()), ntohs(serverRTCPPort.num()), destinationTTL,
-	     fOurSessionId);
   } else {
     switch (streamingMode) {
     case RTP_UDP: {
@@ -966,7 +1004,8 @@ void RTSPServer::RTSPClientSession
 					       fStreamStates[i].streamToken,
 					       (TaskFunc*)noteClientLiveness,
 					       this,
-					       rtpSeqNum, rtpTimestamp);
+					       rtpSeqNum,
+					       rtpTimestamp);
       const char *urlSuffix = fStreamStates[i].subsession->trackId();
       char* prevRTPInfo = rtpInfo;
       unsigned rtpInfoSize = rtpInfoFmtSize
@@ -1171,6 +1210,12 @@ void RTSPServer::RTSPClientSession
 ::livenessTimeoutTask(RTSPClientSession* clientSession) {
   // If this gets called, the client session is assumed to have timed out,
   // so delete it:
+
+  // However, we don't timeout multicast sessions, because to do so would require
+  // closing all client sessions that have requested the stream - not just this one.
+  // Also, the multicast stream itself would usually not be halted, in any case.
+  if (clientSession->isMulticast()) return;
+
 #ifdef DEBUG
   fprintf(stderr, "RTSP client session from %s has timed out (due to inactivity)\n", our_inet_ntoa(clientSession->fClientAddr.sin_addr));
 #endif
