@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2005 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
 // A data structure that represents a session that consists of
 // potentially multiple (audio and/or video) sub-sessions
 // (This data structure is used for media *streamers* - i.e., servers.
@@ -73,7 +73,7 @@ ServerMediaSession::ServerMediaSession(UsageEnvironment& env,
 }
 
 ServerMediaSession::~ServerMediaSession() {
-  delete fSubsessionsHead;
+  Medium::close(fSubsessionsHead);
   delete[] fStreamName;
   delete[] fInfoSDPString;
   delete[] fDescriptionSDPString;
@@ -194,7 +194,7 @@ char* ServerMediaSession::generateSDPDescription() {
   if (fIsSSM) {
     char const* const sourceFilterFmt =
       "a=source-filter: incl IN IP4 * %s\r\n"
-      "a=rtcp:unicast reflection\r\n";
+      "a=rtcp-unicast: reflection\r\n";
     unsigned sourceFilterFmtSize = strlen(sourceFilterFmt)
       + ourIPAddressStrSize;
 
@@ -323,7 +323,7 @@ ServerMediaSubsession::ServerMediaSubsession(UsageEnvironment& env)
 
 ServerMediaSubsession::~ServerMediaSubsession() {
   delete[] (char*)fTrackId;
-  delete fNext;
+  Medium::close(fNext);
 }
 
 char const* ServerMediaSubsession::trackId() {
