@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
 // A WAV audio file source
 // Implementation
 
@@ -33,10 +33,11 @@ WAVAudioFileSource::createNew(UsageEnvironment& env, char const* fileName) {
     WAVAudioFileSource* newSource = new WAVAudioFileSource(env, fid);
     if (newSource != NULL && newSource->bitsPerSample() == 0) {
       // The WAV file header was apparently invalid.
-      delete newSource; newSource = NULL;
+      Medium::close(newSource);
+      break;
     }
 
-    newSource->fFileSize = GetFileSize(fileName, fid);
+    newSource->fFileSize = (unsigned)GetFileSize(fileName, fid);
 
     return newSource;
   } while (0);
@@ -68,6 +69,11 @@ void WAVAudioFileSource::seekToPCMByte(unsigned byteNumber) {
   fseek(fFid, byteNumber, SEEK_SET);
 }
 
+unsigned char WAVAudioFileSource::getAudioFormat() {
+  return fAudioFormat;
+}
+
+
 #define nextc fgetc(fid)
 #define ucEOF ((unsigned char)EOF)
 
@@ -95,7 +101,8 @@ static Boolean skipBytes(FILE* fid, int num) {
 
 WAVAudioFileSource::WAVAudioFileSource(UsageEnvironment& env, FILE* fid)
   : AudioInputDevice(env, 0, 0, 0, 0)/* set the real parameters later */,
-    fFid(fid), fLastPlayTime(0), fWAVHeaderSize(0), fFileSize(0), fScaleFactor(1) {
+    fFid(fid), fLastPlayTime(0), fWAVHeaderSize(0), fFileSize(0), fScaleFactor(1),
+    fAudioFormat(WA_UNKNOWN) {
   // Check the WAV file header for validity.
   // Note: The following web pages contain info about the WAV format:
   // http://www.technology.niagarac.on.ca/courses/comp630/WavFileFormat.html
@@ -118,8 +125,11 @@ WAVAudioFileSource::WAVAudioFileSource(UsageEnvironment& env, FILE* fid)
     if (!get4Bytes(fid, formatLength)) break;
     unsigned short audioFormat;
     if (!get2Bytes(fid, audioFormat)) break;
-    if (audioFormat != 1) { // not PCM - we can't handle this
-      env.setResultMsg("Audio format is not PCM");
+
+    fAudioFormat = (unsigned char)audioFormat;
+    if (fAudioFormat != WA_PCM && fAudioFormat != WA_PCMA && fAudioFormat != WA_PCMU) { 
+      // not PCM/PCMU/PCMA - we can't handle this
+      env.setResultMsg("Audio format is not PCM/PCMU/PCMA");
       break;
     }
     unsigned short numChannels;
@@ -189,6 +199,8 @@ WAVAudioFileSource::~WAVAudioFileSource() {
   CloseInputFile(fFid);
 }
 
+// Note: We should change the following to use asynchronous file reading, #####
+// as we now do with ByteStreamFileSource. #####
 void WAVAudioFileSource::doGetNextFrame() {
   if (feof(fFid) || ferror(fFid)) {
     handleClosure(this);
