@@ -13,7 +13,7 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
-// Copyright (c) 1996-2005, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2007, Live Networks, Inc.  All rights reserved
 // A common framework, used for the "openRTSP" and "playSIP" applications
 // Implementation
 
@@ -688,6 +688,11 @@ int main(int argc, char** argv) {
 	  // For AMR audio streams, we use a special sink that inserts AMR frame hdrs:
 	  fileSink = AMRAudioFileSink::createNew(*env, outFileName,
 						 fileSinkBufferSize, oneFilePerFrame);
+	} else if (strcmp(subsession->mediumName(), "video") == 0 &&
+	    (strcmp(subsession->codecName(), "H264") == 0)) {
+      // For H.264 video stream, we use a special sink that insert start_codes:
+	  fileSink = H264VideoFileSink::createNew(*env, outFileName,
+						 fileSinkBufferSize, oneFilePerFrame);
 	} else {
 	  // Normal case:
 	  fileSink = FileSink::createNew(*env, outFileName,
