@@ -13,7 +13,7 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
-// Copyright (c) 1996-2001, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2007, Live Networks, Inc.  All rights reserved
 // A program that receives and prints SDP/SAP announcements
 // (on the default SDP/SAP directory: 224.2.127.254/9875)
 
@@ -59,6 +59,14 @@ int main(int argc, char** argv) {
       continue;
     }
 
+    // convert "application/sdp\0" -> "application/sdp\0x20"
+    // or all other nonprintable characters to blank, except new line
+    unsigned idx = 8;
+    while (idx < packetSize) {
+      if (packet[idx] < 0x20 && packet[idx] != '\n') packet[idx] = 0x20;
+      idx++;
+    }
+
     packet[packetSize] = '\0'; // just in case
     printf((char*)(packet+8));
   }
