@@ -13,7 +13,7 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
-// Copyright (c) 1996-2005, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2007, Live Networks, Inc.  All rights reserved
 // A test program that reads a MPEG-4 Video Elementary Stream file,
 // and streams it using RTP
 // main program
@@ -23,7 +23,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 #include "GroupsockHelper.hh"
 
 UsageEnvironment* env;
-char const* inputFileName = "test.m4v";
+char const* inputFileName = "test.m4e";
 MPEG4VideoStreamFramer* videoSource;
 RTPSink* videoSink;
 
