@@ -13,7 +13,7 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
-// Copyright (c) 1996-2004, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2007, Live Networks, Inc.  All rights reserved
 // A test program that demonstrates how to stream - via unicast RTP
 // - various kinds of file on demand, using a built-in RTSP server.
 // main program
@@ -33,6 +33,9 @@ Boolean reuseFirstSource = False;
 // change the following "False" to "True":
 Boolean iFramesOnly = False;
 
+static void announceStream(RTSPServer* rtspServer, ServerMediaSession* sms,
+			   char const* streamName, char const* inputFileName); // fwd
+
 int main(int argc, char** argv) {
   // Begin by setting up our usage environment:
   TaskScheduler* scheduler = BasicTaskScheduler::createNew();
@@ -48,7 +51,7 @@ int main(int argc, char** argv) {
 #endif
 
   // Create the RTSP server:
-  RTSPServer* rtspServer = RTSPServer::createNew(*env, 8554, authDB);
+  RTSPServer* rtspServer = RTSPServer::createNew(*env, 8554, authDB, 45);
   if (rtspServer == NULL) {
     *env << "Failed to create RTSP server: " << env->getResultMsg() << "\n";
     exit(1);
@@ -65,7 +68,7 @@ int main(int argc, char** argv) {
   // A MPEG-4 video elementary stream:
   {
     char const* streamName = "mpeg4ESVideoTest";
-    char const* inputFileName = "test.m4v";
+    char const* inputFileName = "test.m4e";
     ServerMediaSession* sms
       = ServerMediaSession::createNew(*env, streamName, streamName,
 				      descriptionString);
@@ -73,11 +76,7 @@ int main(int argc, char** argv) {
 		       ::createNew(*env, inputFileName, reuseFirstSource));
     rtspServer->addServerMediaSession(sms);
 
-    char* url = rtspServer->rtspURL(sms);
-    *env << "\n\"" << streamName << "\" stream, from the file \""
-	 << inputFileName << "\"\n";
-    *env << "Play this stream using the URL \"" << url << "\"\n";
-    delete[] url;
+    announceStream(rtspServer, sms, streamName, inputFileName);
   }
 
   // A MPEG-1 or 2 audio+video program stream:
@@ -94,11 +93,7 @@ int main(int argc, char** argv) {
     sms->addSubsession(demux->newAudioServerMediaSubsession());
     rtspServer->addServerMediaSession(sms);
 
-    char* url = rtspServer->rtspURL(sms);
-    *env << "\n\"" << streamName << "\" stream, from the file \""
-	 << inputFileName << "\"\n";
-    *env << "Play this stream using the URL \"" << url << "\"\n";
-    delete[] url;
+    announceStream(rtspServer, sms, streamName, inputFileName);
   }
 
   // A MPEG-1 or 2 video elementary stream:
@@ -113,11 +108,7 @@ int main(int argc, char** argv) {
 	       ::createNew(*env, inputFileName, reuseFirstSource, iFramesOnly));
     rtspServer->addServerMediaSession(sms);
 
-    char* url = rtspServer->rtspURL(sms);
-    *env << "\n\"" << streamName << "\" stream, from the file \""
-	 << inputFileName << "\"\n";
-    *env << "Play this stream using the URL \"" << url << "\"\n";
-    delete[] url;
+    announceStream(rtspServer, sms, streamName, inputFileName);
   }
 
   // A MP3 audio stream (actually, any MPEG-1 or 2 audio file will work):
@@ -126,7 +117,7 @@ int main(int argc, char** argv) {
   // To also reorder ADUs before streaming, uncomment the following:
 //#define INTERLEAVE_ADUS 1
   // (For more information about ADUs and interleaving,
-  //  see <http://www.live.com/rtp-mp3/>)
+  //  see <http://www.live555.com/rtp-mp3/>)
   {
     char const* streamName = "mp3AudioTest";
     char const* inputFileName = "test.mp3";
@@ -149,11 +140,7 @@ int main(int argc, char** argv) {
 				   useADUs, interleaving));
     rtspServer->addServerMediaSession(sms);
 
-    char* url = rtspServer->rtspURL(sms);
-    *env << "\n\"" << streamName << "\" stream, from the file \""
-	 << inputFileName << "\"\n";
-    *env << "Play this stream using the URL \"" << url << "\"\n";
-    delete[] url;
+    announceStream(rtspServer, sms, streamName, inputFileName);
   }
 
   // A WAV audio stream:
@@ -170,11 +157,7 @@ int main(int argc, char** argv) {
 	       ::createNew(*env, inputFileName, reuseFirstSource, convertToULaw));
     rtspServer->addServerMediaSession(sms);
 
-    char* url = rtspServer->rtspURL(sms);
-    *env << "\n\"" << streamName << "\" stream, from the file \""
-	 << inputFileName << "\"\n";
-    *env << "Play this stream using the URL \"" << url << "\"\n";
-    delete[] url;
+    announceStream(rtspServer, sms, streamName, inputFileName);
   }
 
   // An AMR audio stream:
@@ -188,11 +171,7 @@ int main(int argc, char** argv) {
 		       ::createNew(*env, inputFileName, reuseFirstSource));
     rtspServer->addServerMediaSession(sms);
 
-    char* url = rtspServer->rtspURL(sms);
-    *env << "\n\"" << streamName << "\" stream, from the file \""
-	 << inputFileName << "\"\n";
-    *env << "Play this stream using the URL \"" << url << "\"\n";
-    delete[] url;
+    announceStream(rtspServer, sms, streamName, inputFileName);
   }
 
   // A 'VOB' file (e.g., from an unencrypted DVD):
@@ -209,32 +188,49 @@ int main(int argc, char** argv) {
     sms->addSubsession(demux->newAC3AudioServerMediaSubsession());
     rtspServer->addServerMediaSession(sms);
 
-    char* url = rtspServer->rtspURL(sms);
-    *env << "\n\"" << streamName << "\" stream, from the file \""
-	 << inputFileName << "\"\n";
-    *env << "Play this stream using the URL \"" << url << "\"\n";
-    delete[] url;
+    announceStream(rtspServer, sms, streamName, inputFileName);
   }
 
   // A MPEG-2 Transport Stream:
   {
     char const* streamName = "mpeg2TransportStreamTest";
     char const* inputFileName = "test.ts";
+    char const* indexFileName = "test.ts";
     ServerMediaSession* sms
       = ServerMediaSession::createNew(*env, streamName, streamName,
 				      descriptionString);
     sms->addSubsession(MPEG2TransportFileServerMediaSubsession
+		       ::createNew(*env, inputFileName, indexFileName, reuseFirstSource));
+    rtspServer->addServerMediaSession(sms);
+
+    announceStream(rtspServer, sms, streamName, inputFileName);
+  }
+
+  // An AAC audio stream (ADTS-format file):
+  {
+    char const* streamName = "aacAudioTest";
+    char const* inputFileName = "test.aac";
+    ServerMediaSession* sms
+      = ServerMediaSession::createNew(*env, streamName, streamName,
+				      descriptionString);
+    sms->addSubsession(ADTSAudioFileServerMediaSubsession
 		       ::createNew(*env, inputFileName, reuseFirstSource));
     rtspServer->addServerMediaSession(sms);
 
-    char* url = rtspServer->rtspURL(sms);
-    *env << "\n\"" << streamName << "\" stream, from the file \""
-	 << inputFileName << "\"\n";
-    *env << "Play this stream using the URL \"" << url << "\"\n";
-    delete[] url;
+    announceStream(rtspServer, sms, streamName, inputFileName);
   }
 
   env->taskScheduler().doEventLoop(); // does not return
 
   return 0; // only to prevent compiler warning
 }
+
+static void announceStream(RTSPServer* rtspServer, ServerMediaSession* sms,
+			   char const* streamName, char const* inputFileName) {
+  char* url = rtspServer->rtspURL(sms);
+  UsageEnvironment& env = rtspServer->envir(); 
+  env << "\n\"" << streamName << "\" stream, from the file \""
+      << inputFileName << "\"\n";
+  env << "Play this stream using the URL \"" << url << "\"\n";
+  delete[] url;
+}
